@@ -1,4 +1,4 @@
-/*  pl-op.c,v 1.6 1994/11/24 14:07:32 jan Exp
+/*  pl-op.c,v 1.7 1995/08/16 11:51:11 jan Exp
 
     Copyright (c) 1990 Jan Wielemaker. All rights reserved.
     See ../LICENCE to find out about your rights.
@@ -303,13 +303,15 @@ static struct operator operators[] = {
   OP(ATOM_hat,		OP_XFY,		200),		/* ^ */
   OP(ATOM_discontiguous,OP_FX,	       1150),		/* discontiguous */
   OP(ATOM_dynamic,	OP_FX,	       1150),		/* dynamic */
+  OP(ATOM_volatile,	OP_FX,	       1150), 		/* volatile */
+  OP(ATOM_initialization,OP_FX,	       1150), 		/* initialization */
   OP(ATOM_is,		OP_XFX,		700),		/* is */
   OP(ATOM_mod,		OP_XFX,		300),		/* mod */
   OP(ATOM_module_transparent,OP_FX,    1150),		/* module_transparent */
   OP(ATOM_multifile,	OP_FX,	       1150),		/* multifile */
   OP(ATOM_not,		OP_FY,		900),		/* not */
   OP(ATOM_xor,		OP_YFX,		400),		/* xor */
-  OP(ATOM_tilde,	OP_FX,		900),		/* ~ */
+/*OP(ATOM_tilde,	OP_FX,		900),*/		/* ~ */
 
   OP((Atom)NULL,	0,		0)
 };
